/******************************************************************************
*                                                                             *
*    ClientContext.cpp                      Copyright(c) 2007-2010 itow,y.    *
*                                                                             *
******************************************************************************/

/*
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include <windows.h>

#include "pntypes.h"
#include "pnwintyp.h"
#include "pncom.h"
#include "rmacomm.h"
#include "rmawin.h"
#include "rmaclsnk.h"
#include "rmaerror.h"

#include "ClientAdviseSink.h"
#include "ErrorSink.h"
#include "SiteSupplier.h"
#include "AuthenticManager.h"
#include "ClientContext.h"




AntClientContext::AntClientContext()
	: m_RefCount(1)
	, m_pClientSink(NULL)
	, m_pErrorSink(NULL)
	, m_pAuthMgr(NULL)
	, m_pSiteSupplier(NULL)
{
}


AntClientContext::~AntClientContext()
{
	Close();
}


void AntClientContext::Init(IUnknown *pUnknown)
{
	m_pErrorSink=new AntErrorSink(pUnknown);
	m_pErrorSink->AddRef();
	m_pClientSink=new AntClientAdviseSink(pUnknown);
	m_pClientSink->AddRef();
	m_pAuthMgr=new AntAuthenticationManager();
	m_pAuthMgr->AddRef();
	m_pSiteSupplier=new AntSiteSupplier(pUnknown);
	m_pSiteSupplier->AddRef();
}


void AntClientContext::Close()
{
	PN_RELEASE(m_pClientSink);
	PN_RELEASE(m_pErrorSink);
	PN_RELEASE(m_pAuthMgr);
	PN_RELEASE(m_pSiteSupplier);
}


HWND AntClientContext::GetWindow()
{
	if (m_pSiteSupplier==NULL)
		return NULL;
	return m_pSiteSupplier->GetWindow();
}


STDMETHODIMP_(ULONG) AntClientContext::AddRef()
{
	return InterlockedIncrement(&m_RefCount);
}


STDMETHODIMP_(ULONG) AntClientContext::Release()
{
	if (InterlockedDecrement(&m_RefCount)>0)
		return m_RefCount;
	delete this;
	return 0;
}


STDMETHODIMP AntClientContext::QueryInterface(REFIID riid,void **ppObj)
{
	if (IsEqualIID(riid,IID_IUnknown)) {
		AddRef();
		*ppObj=static_cast<IUnknown*>(this);
		return PNR_OK;
	} else if (m_pClientSink!=NULL
			&& m_pClientSink->QueryInterface(riid,ppObj)==PNR_OK) {
		return PNR_OK;
	} else if (m_pErrorSink!=NULL
			&& m_pErrorSink->QueryInterface(riid,ppObj)==PNR_OK) {
		return PNR_OK;
	} else if (m_pAuthMgr!=NULL
			&& m_pAuthMgr->QueryInterface(riid,ppObj)==PNR_OK) {
		return PNR_OK;
	} else if (m_pSiteSupplier!=NULL
			&& m_pSiteSupplier->QueryInterface(riid,ppObj)==PNR_OK) {
		return PNR_OK;
	}
	*ppObj=NULL;
	return PNR_NOINTERFACE;
}
