/******************************************************************************
*                                                                             *
*    DropTarget.cpp                         Copyright(c) 2007-2010 itow,y.    *
*                                                                             *
******************************************************************************/

/*
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include <windows.h>
#include <shlobj.h>
#include <shlwapi.h>
#include <tchar.h>
#include "Ant-RP.h"
#include "DropTarget.h"




static bool IsValidPath(LPCTSTR pszPath)
{
	return (IsCharAlpha(pszPath[0]) && pszPath[1]==':')
								|| PathIsURL(pszPath) || PathIsUNC(pszPath);
}


AntDropTarget::AntDropTarget()
	: m_RefCount(1)
{
}


AntDropTarget::~AntDropTarget()
{
}


STDMETHODIMP AntDropTarget::QueryInterface(REFIID riid,void **ppvObj)
{
	if (IsEqualIID(riid,IID_IUnknown)) {
		*ppvObj=static_cast<IUnknown*>(this);
		AddRef();
		return S_OK;
	} else if (IsEqualIID(riid,IID_IDropTarget)) {
		*ppvObj=static_cast<IDropTarget*>(this);
		AddRef();
		return S_OK;
	}
	*ppvObj=NULL;
	return E_NOINTERFACE;
}


STDMETHODIMP_(ULONG) AntDropTarget::AddRef()
{
	InterlockedIncrement(&m_RefCount);
	return m_RefCount;
}


STDMETHODIMP_(ULONG) AntDropTarget::Release()
{
	if (InterlockedDecrement(&m_RefCount)==0)
		delete this;
	return m_RefCount;
}


STDMETHODIMP AntDropTarget::DragEnter(IDataObject *pDataObject,
								DWORD grfKeyState,POINTL ptl,DWORD *pdwEffect)
{
	m_pDataObject=pDataObject;
	return DragOver(grfKeyState,ptl,pdwEffect);
}


STDMETHODIMP AntDropTarget::DragOver(DWORD /*grfKeyState*/,POINTL /*ptl*/,
															DWORD *pdwEffect)
{
	bool fOK=false;
	FORMATETC ft;

#ifndef UNICODE
	ft.cfFormat=CF_TEXT;
#else
	ft.cfFormat=CF_UNICODETEXT;
#endif
	ft.ptd=NULL;
	ft.dwAspect=DVASPECT_CONTENT;
	ft.lindex=-1;
	ft.tymed=TYMED_HGLOBAL;
	if (m_pDataObject->QueryGetData(&ft)==S_OK) {
		STGMEDIUM sg;

		if (m_pDataObject->GetData(&ft,&sg)==S_OK) {
			LPCTSTR pszText;

			pszText=(LPCTSTR)GlobalLock(sg.hGlobal);
			fOK=pszText!=NULL && IsValidPath(pszText);
			GlobalUnlock(sg.hGlobal);
			ReleaseStgMedium(&sg);
		}
	}
	*pdwEffect=fOK?DROPEFFECT_LINK:DROPEFFECT_NONE;
	//SetCursor(LoadCursor(NULL,fOK?IDC_ARROW:IDC_NO));
	return S_OK;
}


STDMETHODIMP AntDropTarget::DragLeave()
{
	m_pDataObject=NULL;
	//SetCursor(LoadCursor(NULL,IDC_NO));
	return S_OK;
}


STDMETHODIMP AntDropTarget::Drop(IDataObject *pDataObject,
						DWORD /*grfKeyState*/,POINTL /*ptl*/,DWORD* pdwEffect)
{
	bool fOK=false;
	FORMATETC ft;
	STGMEDIUM sg;

#ifndef UNICODE
	ft.cfFormat=CF_TEXT;
#else
	ft.cfFormat=CF_UNICODETEXT;
#endif
	ft.ptd=NULL;
	ft.dwAspect=DVASPECT_CONTENT;
	ft.lindex=-1;
	ft.tymed=TYMED_HGLOBAL;
	if (pDataObject->GetData(&ft,&sg)==S_OK) {
		LPCTSTR pszText;

		pszText=(LPCTSTR)GlobalLock(sg.hGlobal);
		if (pszText!=NULL && IsValidPath(pszText)) {
			SendMessage(GetMainWindow(),ANT_MESSAGE_OPEN,0,(LPARAM)pszText);
			fOK=true;
		}
		GlobalUnlock(sg.hGlobal);
		ReleaseStgMedium(&sg);
	}
	*pdwEffect=fOK?DROPEFFECT_LINK:DROPEFFECT_NONE;
	return S_OK;
}
