/******************************************************************************
*                                                                             *
*    SiteSupplier.cpp                       Copyright(c) 2007-2010 itow,y.    *
*                                                                             *
******************************************************************************/

/*
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include <windows.h>

#include "pncom.h"
#include "pntypes.h"
#include "pnwintyp.h"
#include "rmawin.h"
#include "rmapckts.h"
#include "rmacomm.h"

#include "Ant-RP.h"
#include "SiteSupplier.h"




AntSiteSupplier::AntSiteSupplier(IUnknown *pUnkPlayer)
	: m_RefCount(0)
	, m_pUnkPlayer(pUnkPlayer)
	, m_pSiteManager(NULL)
	, m_pCCF(NULL)
	, m_hwnd(NULL)
{
	if (pUnkPlayer!=NULL) {
		m_pUnkPlayer->QueryInterface(IID_IRMASiteManager,
													(void**)&m_pSiteManager);
		m_pUnkPlayer->QueryInterface(IID_IRMACommonClassFactory,
															(void**)&m_pCCF);
		m_pUnkPlayer->AddRef();
	}
}


AntSiteSupplier::~AntSiteSupplier()
{
	PN_RELEASE(m_pSiteManager);
	PN_RELEASE(m_pCCF);
	PN_RELEASE(m_pUnkPlayer);
}


LRESULT CALLBACK SiteHookProc(HWND hwnd,UINT uMsg,WPARAM wParam,LPARAM lParam)
{
	switch (uMsg) {
	case WM_SIZE:
		{
			IRMASite *pSite=(IRMASite*)GetProp(hwnd,TEXT("Site"));
			PNxSize Size;

			Size.cx=LOWORD(lParam);
			Size.cy=HIWORD(lParam);
			pSite->SetSize(Size);
		}
		break;

	case WM_RBUTTONDOWN:
		{
			HWND hwndParent=GetParent(hwnd);
			POINT pt;

			pt.x=LOWORD(lParam);
			pt.y=HIWORD(lParam);
			MapWindowPoints(hwnd,hwndParent,&pt,1);
			SendMessage(hwndParent,uMsg,wParam,MAKELPARAM(pt.x,pt.y));
		}
		return 0;

	case WM_SETCURSOR:
		SetCursor(LoadCursor(NULL,IDC_ARROW));
		return TRUE;
	}
	return CallWindowProc((WNDPROC)GetProp(hwnd,TEXT("OldWndProc")),hwnd,uMsg,
																wParam,lParam);
}


STDMETHODIMP AntSiteSupplier::SitesNeeded(UINT32 RequestID,IRMAValues *pProps)
{
	if (pProps==NULL)
		return PNR_INVALID_PARAMETER;

	HRESULT hres;
	IRMASiteWindowed *pSiteWindowed;
	IRMASite *pSite;
	IRMAValues *pSiteProps;
	IRMABuffer *pValue=NULL;
	PNxWindow *pWindow;

	hres=m_pCCF->CreateInstance(CLSID_IRMASiteWindowed,(void**)&pSiteWindowed);
	if (hres!=PNR_OK)
		goto exit;
	hres=pSiteWindowed->QueryInterface(IID_IRMASite,(void**)&pSite);
	if (hres!=PNR_OK)
		goto exit;
	hres=pSiteWindowed->QueryInterface(IID_IRMAValues,(void**)&pSiteProps);
	if (hres!=PNR_OK)
		goto exit;
	hres=pProps->GetPropertyCString("playto",pValue);
	if (hres==PNR_OK) {
		pSiteProps->SetPropertyCString("channel",pValue);
		PN_RELEASE(pValue);
	} else {
		hres=pProps->GetPropertyCString("name",pValue);
		if (hres==PNR_OK) {
			pSiteProps->SetPropertyCString("LayoutGroup",pValue);
			PN_RELEASE(pValue);
		}
	}
	hres=pSiteWindowed->Create(GetScreenWindow(),WS_CHILD | WS_VISIBLE);
	if (hres!=PNR_OK)
		goto exit;
	pWindow=pSiteWindowed->GetWindow();
	m_hwnd=(HWND)pWindow->window;
	SetProp(m_hwnd,TEXT("OldWndProc"),
		(WNDPROC)SetWindowLongPtr(m_hwnd,GWLP_WNDPROC,(LONG_PTR)SiteHookProc));
	SetProp(m_hwnd,TEXT("Site"),pSite);
	pSite->AttachWatcher((IRMASiteWatcher*)this);
	hres=m_pSiteManager->AddSite(pSite);
	if (hres!=PNR_OK)
		goto exit;
	m_CreatedSites.SetAt((void*)RequestID,pSite);
	pSite->AddRef();
exit:
	PN_RELEASE(pSiteProps);
	PN_RELEASE(pSiteWindowed);
	PN_RELEASE(pSite);
	return hres;
}


STDMETHODIMP AntSiteSupplier::SitesNotNeeded(UINT32 RequestID)
{
	void *pVoid;
	IRMASite *pSite;
	IRMASiteWindowed *pSiteWindowed;

	if (!m_CreatedSites.Lookup((void*)RequestID,pVoid))
		return PNR_INVALID_PARAMETER;
	pSite=(IRMASite*)pVoid;
	m_pSiteManager->RemoveSite(pSite);
	pSite->DetachWatcher();
	pSite->QueryInterface(IID_IRMASiteWindowed,(void**)&pSiteWindowed);
	pSiteWindowed->Destroy();
	pSiteWindowed->Release();
	pSite->Release();
	m_CreatedSites.RemoveKey((void*)RequestID);
	m_hwnd=NULL;
	return PNR_OK;
}


STDMETHODIMP AntSiteSupplier::BeginChangeLayout()
{
	return PNR_OK;
}


STDMETHODIMP AntSiteSupplier::DoneChangeLayout()
{
	return PNR_OK;
}


STDMETHODIMP AntSiteSupplier::AttachSite(IRMASite * /*pSite*/)
{
	return PNR_OK;
}


STDMETHODIMP AntSiteSupplier::DetachSite()
{
	return PNR_OK;
}


STDMETHODIMP AntSiteSupplier::ChangingPosition(PNxPoint /*posOld*/,
													REF(PNxPoint) /*posNew*/)
{
	return PNR_OK;
}


STDMETHODIMP AntSiteSupplier::ChangingSize(PNxSize /*sizeOld*/,
													REF(PNxSize) /*sizeNew*/)
{
	return PNR_OK;
}


STDMETHODIMP_(ULONG) AntSiteSupplier::AddRef()
{
	return InterlockedIncrement(&m_RefCount);
}


STDMETHODIMP_(ULONG) AntSiteSupplier::Release()
{
	if (InterlockedDecrement(&m_RefCount)>0)
		return m_RefCount;
	delete this;
	return 0;
}


STDMETHODIMP AntSiteSupplier::QueryInterface(REFIID riid,void **ppObj)
{
	if (IsEqualIID(riid,IID_IUnknown)) {
		AddRef();
		*ppObj=static_cast<IUnknown*>(static_cast<IRMASiteSupplier*>(this));
		return PNR_OK;
	} else if (IsEqualIID(riid,IID_IRMASiteSupplier)) {
		AddRef();
		*ppObj=static_cast<IRMASiteSupplier*>(this);
		return PNR_OK;
	} else if (IsEqualIID(riid,IID_IRMASiteWatcher)) {
		AddRef();
		*ppObj=static_cast<IRMASiteWatcher*>(this);
		return PNR_OK;
	}
	*ppObj=NULL;
	return PNR_NOINTERFACE;
}
