/******************************************************************************
*                                                                             *
*    TACManager.h                           Copyright(c) 2007-2010 itow,y.    *
*                                                                             *
******************************************************************************/

/*
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include <windows.h>

#include "pncom.h"
#include "pntypes.h"
#include "pnwintyp.h"
#include "rmawin.h"
#include "rmacomm.h"
#include "rmamon.h"
#include "rmaengin.h"
#include "rmacore.h"
#include "rmapckts.h"

#include "TACManager.h"


static const char *szTACNames[] = {
	"Title",
	"Author",
	"Copyright",
	"Abstract",
	"Keywords"
};




AntTACManager::AntTACManager()
	: m_lRefCount(0)
	, m_pTitle(NULL)
	, m_pAuthor(NULL)
	, m_pCopyright(NULL)
	, m_pAbstract(NULL)
	, m_pKeywords(NULL)
	, m_pRegName(NULL)
	, m_uPlayerIndex(0)
	, m_pPropWatch(NULL)
	, m_pRegistry(NULL)
{
	for (int i=0;i<NUM_TAC_NAMES;i++)
		m_uTACPropIDs[i]=0;
}


AntTACManager::~AntTACManager()
{
	delete [] m_pTitle;
	delete [] m_pAuthor;
	delete [] m_pCopyright;
	delete [] m_pAbstract;
	delete [] m_pKeywords;
	delete [] m_pRegName;
}


BOOL AntTACManager::Init(IRMAPlayer *pPlayer)
{
	BOOL bOk=FALSE;

	if (pPlayer!=NULL) {
		IRMARegistryID *pRegID=NULL;
		if (pPlayer->QueryInterface(IID_IRMARegistryID,(void**)&pRegID)==
																	PNR_OK) {
			pRegID->GetID(m_uPlayerIndex);
			pRegID->Release();
			if (pPlayer->QueryInterface(IID_IRMAPNRegistry,
											(void**)&m_pRegistry)==PNR_OK) {
				IRMABuffer *pBuffer=NULL;
				if (m_pRegistry->GetPropName(m_uPlayerIndex,pBuffer)==PNR_OK) {
					ULONG32 uLen=pBuffer->GetSize();

					m_pRegName=new char[uLen+1];
					lstrcpyA(m_pRegName,(const char*)pBuffer->GetBuffer());
					pBuffer->Release();
					if (m_pRegistry->CreatePropWatch(m_pPropWatch)==PNR_OK) {
						m_pPropWatch->Init((IRMAPropWatchResponse*)this);
						bOk=TRUE;
						if (m_pRegName && *m_pRegName!='\0') {
							for (int i=0;i<NUM_TAC_NAMES;i++) {
								UINT32 uId=m_uTACPropIDs[i];
								char szName[MAX_DISPLAY_NAME];

								wsprintfA(szName,"%s.%s",
													m_pRegName,szTACNames[i]);
								m_uTACPropIDs[i]=m_pRegistry->GetId(szName);
								if (uId>0)
									m_pPropWatch->SetWatchById(uId);
							}
						}
						Update();
		 			}
				}
			}
		}
	}
	return bOk;
}


void AntTACManager::Terminate()
{
	if (m_pPropWatch!=NULL) {
		for (int i=0;i<NUM_TAC_NAMES;i++) {
			if (m_uTACPropIDs[i]>0) {
				m_pPropWatch->ClearWatchById(m_uTACPropIDs[i]);
				m_uTACPropIDs[i]=0;
			}
		}
		m_pPropWatch->Release();
		m_pPropWatch=NULL;
	}
	if (m_pRegistry!=NULL) {
		m_pRegistry->Release();
		m_pRegistry = NULL;
	}
}


const char *AntTACManager::GetTitle()
{
	return m_pTitle;
}


const char *AntTACManager::GetAuthor()
{
	return m_pAuthor;
}


const char *AntTACManager::GetCopyright()
{
	return m_pCopyright;
}


const char *AntTACManager::GetAbstract()
{
	return m_pAbstract;
}


const char *AntTACManager::GetKeywords()
{
	return m_pKeywords;
}


void AntTACManager::SetStringProp(char **ppProp,const char *pNew,UINT32 uLen)
{
	if (*ppProp!=NULL) {
		if ((UINT32)lstrlenA(*ppProp)<uLen+1) {
			delete [] *ppProp;
			*ppProp=NULL;
		}
	}
	if (*ppProp==NULL)
		*ppProp=new char[uLen+1];
	lstrcpynA(*ppProp,pNew,uLen);
}


void AntTACManager::Update()
{
	if (m_pRegistry!=NULL) {
		for (int i=0;i<NUM_TAC_NAMES;i++) {
			if (m_uTACPropIDs[i]>0) {
				IRMABuffer *pValue=NULL;

				m_pRegistry->GetStrById(m_uTACPropIDs[i],pValue);
				if (pValue!=NULL) {
					const char *pProp=(const char*)pValue->GetBuffer();
					UINT32 uLen=pValue->GetSize();

					switch (i) {
					case TAC_TITLE:
						SetStringProp(&m_pTitle,pProp,uLen);
						break;
					case TAC_AUTHOR:
						SetStringProp(&m_pAuthor,pProp,uLen);
						break;
					case TAC_COPYRIGHT:
						SetStringProp(&m_pCopyright,pProp,uLen);
						break;
					case TAC_ABSTRACT:
						SetStringProp(&m_pAbstract,pProp,uLen);
						break;
					case TAC_KEYWORDS:
						SetStringProp(&m_pKeywords,pProp,uLen);
						break;
					}
					pValue->Release();
		 			pValue = NULL;
				}
			}
		}
	}
}


STDMETHODIMP AntTACManager::AddedProp(const UINT32 /*id*/,
				const RMAPropType /*propType*/,const UINT32 /*ulParentHash*/)
{
	return PNR_NOTIMPL;
}


STDMETHODIMP AntTACManager::ModifiedProp(const UINT32 /*id*/,
				const RMAPropType /*propType*/,const UINT32 /*ulParentHash*/)
{
	return PNR_NOTIMPL;
}


STDMETHODIMP AntTACManager::DeletedProp(const UINT32 id,
												const UINT32 /*ulParentHash*/)
{
	for (int i=0;i<NUM_TAC_NAMES;i++) {
		if (m_uTACPropIDs[i]==id)
			m_uTACPropIDs[i]=0;
	}
	return m_pPropWatch->ClearWatchById(id);
}


STDMETHODIMP_(ULONG) AntTACManager::AddRef()
{
	return InterlockedIncrement(&m_lRefCount);
}


STDMETHODIMP_(ULONG) AntTACManager::Release()
{
	if (InterlockedDecrement(&m_lRefCount)>0)
		return m_lRefCount;
	delete this;
	return 0;
}


STDMETHODIMP AntTACManager::QueryInterface(REFIID iid,void **ppObj)
{
	if (IsEqualIID(iid,IID_IUnknown)) {
		AddRef();
		*ppObj=static_cast<IUnknown*>(this);
		return PNR_OK;
	} else if (IsEqualIID(iid,IID_IRMAPropWatchResponse)) {
		AddRef();
		*ppObj=static_cast<IRMAPropWatchResponse*>(this);
		return PNR_OK;
	}
	*ppObj=NULL;
	return PNR_NOINTERFACE;
}
